@@ -1,8 +1,9 @@
-pub struct Matrix<T: Clone + Copy> {
+#[derive(Debug)]
+pub struct Matrix<T: Clone> {
     matrix: Vec<Vec<Option<T>>>,
 }
 
-impl<T: Clone + Copy> Matrix<T> {
+impl<T: Clone> Matrix<T> {
     pub fn new(size: (usize, usize)) -> Matrix<T> {
         let (m, n) = size;
         let mut matrix: Vec<Vec<Option<T>>> = Vec::new();
@@ -16,10 +17,16 @@ impl<T: Clone + Copy> Matrix<T> {
         return Matrix::<T> { matrix };
     }
 
-    pub fn get(&self, pos: (usize, usize)) -> Option<T> {
+    pub fn get(&mut self, pos: (usize, usize)) -> Option<T> {
         let (m, n) = pos;
 
-        return self.matrix[m][n];
+        let item = self.matrix[m][n].clone();
+
+        if item.is_some() {
+            self.clear(pos);
+        }
+
+        return item;
     }
 
     pub fn get_ref(&self, pos: (usize, usize)) -> Option<&T> {
@@ -38,6 +45,25 @@ impl<T: Clone + Copy> Matrix<T> {
         let (m, n) = pos;
         self.matrix[m][n] = Some(item);
     }
+
+    pub fn clear(&mut self, pos: (usize, usize)) {
+        let (m, n) = pos;
+
+        self.matrix[m][n] = None;
+    }
+
+    pub fn translate(&mut self, from: (usize, usize), dir: (usize, usize)) -> Result<(), ()> {
+        let item = self.get(from);
+
+        if item.is_none() {
+            return Err(());
+        }
+
+        self.set(item.unwrap(), dir);
+        self.clear(from);
+
+        return Ok(());
+    }
 }
 
 impl<T: Clone + Copy> From<Vec<Vec<T>>> for Matrix<T> {
@@ -54,4 +80,4 @@ impl<T: Clone + Copy> From<Vec<Vec<T>>> for Matrix<T> {
 
         return matrix;
     }
-}
+}
\ No newline at end of file
